% Compute the logistic loss for multi-class classification, and its gradient 
% X: num_fea * num_ex
% Ymat: num_class * num_ex (a single 1 in each column to indicate the class)
% W: num_fea * num_class
function [f, G, S] = logistic_multiclass(X, Ymat, par_smooth, W, flag)
            
  if nargin < 5, flag = [];   end
  num_ex = size(Ymat, 2);
  t = cputime;

  if strcmp(flag, 'fdual')  % dual objective evaluated at X        
    error('fdual not implemented yet for logistic_multiclass');    
  end
  
  if strcmp(flag, 'dirHes')
    f = []; G = [];    
    model = W{1};
    searchDir = W{2};   % = U B V'
    if isnumeric(X)
      output = model.V*(model.B*(model.U'*X));  % output: num_class * num_ex 
      C = model.V*(model.B*(searchDir.U'*X)) ...
            + searchDir.V*(model.B*(model.U'*X)) ...
            + model.V*(searchDir.B*(model.U'*X)); % num_class * num_ex
    else
      eval(sprintf('global %s', X));
      eval(['output = model.V*(model.B*(model.U''*' X '));']);
      line1 = ['C = model.V*(model.B*(searchDir.U''*' X '))'];
      line2 = ['+searchDir.V*(model.B*(model.U''*' X '))'];
      line3 = ['+model.V*(searchDir.B*(model.U''*' X '));'];
      eval([line1 line2 line3]);
    end
    if par_smooth.wgt > 1e-7
      D = model.U*(model.B*searchDir.V') + searchDir.U*(model.B*model.V') ...
            + model.U*(searchDir.B*model.V');
    end
    max_output = max(output);
    exp_diff = exp(bsxfun(@minus, output, max_output));
    P = bsxfun(@rdivide, exp_diff, sum(exp_diff));
    C = C .* P;
    C = C - bsxfun(@times, P, sum(C));
%     S = {X, C'/num_ex};
    if par_smooth.wgt > 1e-7
      W = model.U*model.B*model.V';
      [~, ~, H] = smooth_L11(W, sparse(size(W,1),size(W,2)), par_smooth.sigma);
      S.b = {X, C'/num_ex};
      S.a = par_smooth.wgt * D .* H;
    else
      S = {X, C'/num_ex};
    end
    return;
  end
  
  if iscell(W)
    if isnumeric(X)
      output = W{2} * (W{1}'*X);  % output: num_class * num_ex
    else
      eval(['global ' X]);
      eval(['output = W{2} * (W{1}''*' X ');']);
    end
  else
    if isnumeric(X)
      output = W'*X;
    else 
      eval(['global ' X]);
      eval(['output = W''*' X ';']);
    end
  end
  max_output = max(output);
  exp_diff = exp(bsxfun(@minus, output, max_output));
  sum_exp_diff = sum(exp_diff);
  class_output = output(Ymat)';  % discriminant value for the ground truth
                                 % take one element each column, then concatenate into a column vector
  if strcmp(flag, 'eval')    
    class_else_output = max(reshape(output(~Ymat), size(Ymat,1)-1, num_ex));
    f1 = 100*sum(class_output - class_else_output > 1e-5)/num_ex;
%     num_err = sum(abs(max_output - class_output) > 1e-5);
%     f = 100*(1 - num_err/num_ex); % accuracy
    f2 = 100 - f1;
    f.l1 = f1;    f.l2 = f2;
    return;
  end
  
  f = sum(log(sum_exp_diff) + max_output - class_output) / num_ex;
  
  if iscell(W),   W = W{1} * W{2}';    end
  if nargout < 2
    f = f + par_smooth.wgt*smooth_L11(W, sparse(size(W,1),size(W,2)), par_smooth.sigma);
  else
    % Compute the gradient in smoothing ||W||_1,1
    if par_smooth.wgt > 1e-7
      [f1, GG] = smooth_L11(W, sparse(size(W,1),size(W,2)), par_smooth.sigma);
      f = f + par_smooth.wgt*f1;
    end
    
    % C is num_class * num_ex
    C = bsxfun(@rdivide, exp_diff, sum_exp_diff*num_ex) - Ymat/num_ex;
    if par_smooth.wgt > 1e-7
      G.a = par_smooth.wgt*GG;
      G.b = {X, C'};
    else
      G = {X, C'};
    end
  end
  
  global feval_logistic time_logistic
  time_logistic = time_logistic + (cputime - t);
  feval_logistic = feval_logistic + 1;
end
