function [f, G, S] = matrix_completion(data, X, flag)

  if nargin < 3, flag = [];   end
  d1 = data.d1;   d2 = data.d2;
%   num_obs = nnz(data.entries);
  num_obs = 1;

  if strcmp(flag, 'fdual')
    % This X is gradient computed in the later part of this function
    f = norm(X, 'fro')^2/4 + nonzeros(X)'*data.entries;
    return;

  elseif strcmp(flag, 'dirHes')
    searchDir = X{2};
    model = X{1};
    total = proj_omega_sub(searchDir.U*model.B, model.V, data.jc, data.ir);
    total = total + proj_omega_sub(model.U*searchDir.B, model.V, data.jc, data.ir);
    total = total + proj_omega_sub(model.U*model.B, searchDir.V, data.jc, data.ir);
    S = make_spmat(data.jc, data.ir, 2 * total, d1, d2);
    f = []; G = [];
    return;
  end

  if isa(X, 'cell')
    if size(X{1}, 2) ~= size(X{2}, 2)
      error('Wrong size in loss/matrix_completion\n');
    end

    errors = proj_omega_sub(X{1}, X{2}, data.jc, data.ir) - data.entries;
  else                
    errors = proj_omega_sub_single(X, data.jc, data.ir) - data.entries;
  end

  if strcmp(flag, 'eval')
    f.l1 = mean(abs(errors));
    f.l2 = norm(errors) / sqrt(length(errors));
    return;
  end

  f = norm(errors)^2/num_obs;

  if nargout > 1
    errors = (2/num_obs)*errors;
    G = make_spmat(data.jc, data.ir, errors, d1, d2);
  end
end
