gfunction [sol, obj, iter, msg, outputs] = ...
          multiview_alt_free(lambda, X0, evaluator, opts)

% Solve  Min_X  func(X) + lambda * ||X||_*

m = size(X0, 1);
n = size(X0, 2);
k = opts.init_rank;  % Initialize U and V with rank k

U = randn(m, k);
V = randn(k, n);

make_output = (nargout == 5);

num_f1 = opts.num_f1;
num_f2 = opts.num_f2;
l = num_f1 + num_f2;
num_class = opts.num_class;
w1 = opts.weight_X1;
w2 = opts.weight_X2;
w3 = opts.weight_X3;
w = [w1, w2, w3];
ub1 = opts.ub(1);
ub2 = opts.ub(2);
assert(m == num_f1 + num_f2 + num_class);

if make_output
  outputs = [];
  outputs.obj_list = zeros(opts.max_iter, 1);
  outputs.time_list = zeros(opts.max_iter, 1);
  outputs.loss_list = zeros(opts.max_iter, 1);
  outputs.perf_list = zeros(opts.max_iter, 1);  
end

total_time = 0;     % Total time consumed by the algorithm
local_search_time = 0;  % Time spend on local search

global feval_logistic
global time_logistic

feval_logistic = 0;
time_logistic = 0;

perf = 0;
% Randomly initialize the bases
U1 = randn(num_f1, k);
U1 = U1 ./ repmat(sqrt(sum(U1.^2)*ub1), [num_f1, 1]);
U2 = randn(num_f2, k);
U2 = U2 ./ repmat(sqrt(sum(U2.^2)*ub2), [num_f2, 1]);
U3 = 2*(rand(num_class, k) - 0.5);
U = [U1; U2; U3];

for i = 1 : opts.max_iter
      
  t1 = cputime;  
  
  % Fix bases, optimize the latent V
  n1 = sum(U(1:num_f1,:).^2);
  n2 = sum(U(1+num_f1:l,:).^2);
  nsq_list = U(1+l:end,:).^2;
  const_U_obj = sum(max([n1/ub1^2; n2/ub2^2; nsq_list]));
  
  ub = Inf(n*k, 1);
  [V, obj, iter, feval, msg] = lbfgsb(V(:), -ub, ub, @obj_V, ...
                                              [], [], opts.lbfgsb_in);
  V = reshape(V, [k, n]);                                            
                                            
  % Fix V, optimize the bases
  const_V_obj = norm(V, 'fro')^2;
  ub = Inf(numel(U), 1);
  [U, obj, iter, feval, msg] = lbfgsb(U(:), -ub, ub, @obj_U, ...
                                              [], [], opts.lbfgsb_in);
  U = reshape(U, [num_class+l, k]);
  
  if opts.use_local || i > 1    
    fprintf('iter=%d, loss = %g, obj=%g, feval=%d, teval=%g, k=%d, time=%g, msg=%s\n', ...
          i, loss, obj, feval_logistic, time_logistic, k, total_time, msg2str(msg, 'lbfgsb'));
  end
  
  if i > 1 && abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
    msg = 'Stop with small relative change';    
%     disp(msg);
    break;
  else
    pre_obj = obj;
  end

  
  t1 = cputime - t1;  total_time = total_time + t1;
  
  if ~isempty(evaluator)
    perf = evaluator([{U}, {V}]);
  end
  
  if make_output
    outputs.obj_list(i) = obj;
    outputs.time_list(i) = total_time;
    outputs.loss_list(i) = loss;
    outputs.perf_list(i) = perf;
  end
end

if make_output
  outputs.obj_list = outputs.obj_list(1:i-1);
  outputs.time_list = outputs.time_list(1:i-1);
  outputs.loss_list = outputs.loss_list(1:i-1);
  outputs.perf_list = outputs.perf_list(1:i-1);
end

sol = U*V;
iter = i;
if i == opts.max_iter
  msg = 'Stop with max iteration';
end  
  
  function [f, g] = obj_V(x)
    VV = reshape(x, [k, n]);
    [G1, G2, grads, f] = comp_grad_mv(U*VV, opts);
    
    f = w*f + 0.5*lambda*(norm(x)^2 + const_U_obj);
        
    G = [G1*w1; G2*w2; grads*w3];
    g = vec(U'*G+lambda*VV);    
  end

  function [f, g] = obj_U(x)
    UU = reshape(x, [m, k]);
    [G1, G2, grads, f] = comp_grad_mv(UU*V, opts);
    
    n1 = sum(UU(1:num_f1,:).^2);
    n2 = sum(UU(num_f1+1:l,:).^2);
    nsq_list = UU(l+1:end,:).^2;
    
    loss = w*f; 
    f = w*f + 0.5*lambda*(const_V_obj + sum(max([n1/ub1^2; n2/ub2^2; nsq_list])));
       
    G = [G1*w1; G2*w2; grads*w3];
    range1 = 1:num_f1;
    range2 = num_f1+1:l;
    
    G_U = zeros(m, k);
    for j = 1 : k
      [nsq_max, nsq_idx] = max(nsq_list(:,j));
      if n1(j) >= max(n2(j), nsq_max)
        G_U(range1, j) = (lambda/ub1^2)*UU(range1, j);
      elseif n2(j) >= max(n1(j), nsq_max)
        G_U(range2, j) = (lambda/ub2^2)*UU(range2, j);
      else
        range3 = l+nsq_idx(1);
        G_U(range3, j) = lambda*UU(l+nsq_idx(1), j);
      end
    end
    
    g = vec(G*V'+G_U);
  end

end
