function [obj, y, Z] = solve_SD(ctx, opts)

if ~isfield(ctx, 'precomp')
  A = ctx.A * ctx.ub(1);
  B = ctx.B * ctx.ub(2);
  AtA = ctx.A'*ctx.A;
  BtB = ctx.B'*ctx.B;
  AtB = ctx.A'*ctx.B;
else
  A = ctx.A;
  B = ctx.B;
  AtA = ctx.AtA;
  BtB = ctx.BtB;
  AtB = ctx.AtB;
end
c = ctx.c;
Atc = A'*c;
Btc = B'*c;

d1 = size(A, 2);
d2 = size(B, 2);
dim = d1 + d2 + 1;

f = [-1; 0; 0];
lp_A = [];
lp_b = [];
lb = [-opts.estimate; 0; 0];
ub = [0; 10000; 10000];

options = optimset('display','off');
for i = 1 : opts.cp_max_iter
  y = linprog(f, lp_A, lp_b, [], [], lb, ub, [], options);   
  [w b flag] = get_cut_plane(y);
  if ~flag, break;  end
  
  lp_A = [lp_A; w];
  lp_b = [lp_b; b];
end

% Return result

obj = -y(1);
% fprintf('SD = %g\n', obj);

if nargout > 2
  Z = -[0, Atc', Btc'; ...
        Atc, AtA, AtB; ...
        Btc, AtB', BtB];
  Z(1,1) = Z(1,1) - sum(y);
  Z(2:d1+1, 2:d1+1) = Z(2:d1+1, 2:d1+1) + y(2)*eye(d1);
  Z(d1+2:end, d1+2:end) = Z(d1+2:end, d1+2:end) + y(3)*eye(d2);
end

  function [w b flag] = get_cut_plane(y)    
    [u, lam] = laneig(@Z_times_vec, dim, 1, 'AS');
       
%     fprintf('iter=%d: min_eig = %g, (%s)\n', i, lam, num2str(y'));
    if lam > -opts.cp_eig_tol
      w = [];   b = [];   flag = false;
    else
      u_M0_u = u'*M0_times_vec(u);
      u_I00_u = u(1)^2;
      u_M1_u = norm(u(2:d1+1))^2 - u(1)^2;
      u_M2_u = norm(u(2+d1:end))^2 - u(1)^2;
      w = [u_I00_u, -u_M1_u, -u_M2_u];
      b = u_M0_u; 
      flag = true;
    end
  end

  function [output] = M0_times_vec(input)
    xx = input(1);
    yy = input(2:d1+1);
    zz = input(d1+2:end);

    t = xx*c + A*yy + B*zz;

    output = -[Atc'*yy + Btc'*zz; ...
               A'*t; ...
               B'*t];
  end

  function [output] = Z_times_vec(input)
    output = M0_times_vec(input);
    output(1) = output(1) - sum(y) * input(1);
    output(2:d1+1) = output(2:d1+1) + y(2)*input(2:d1+1);
    output(d1+2:end) = output(d1+2:end) + y(3)*input(d1+2:end);
  end

end
