
clear; close all; clc;
sd = 1;   rng('default');   rng(sd);
init_boost;

warning off;

plot_fig = 1;
sel = [];
sel.use_LOB = 1;
sel.use_DRTR = 1;
sel.use_r1d = 0;
sel.use_SDP = 0;

load school_b;
load school_1_indexes;
dataset = 'school_b';

[ds_tr, ds_te] = preprop_multitask(x, y, tr, tr_indexes, tst, tst_indexes);

d1 = size(x, 1);
d2 = length(tr_indexes);

%% Setup

exp_param = [];
exp_param.sd = sd;
exp_param.sel = sel;
exp_param.dataset = dataset;
exp_param.plot_fig = plot_fig;
exp_param.obj_func = @(varargin) logistic_multitask(ds_tr, varargin{:});
exp_param.eval_func = @(W) logistic_multitask(ds_te, W, 'eval');

X0 = sparse(d1, d2);

rtol = 1e-4;
init_rank = 1;
opts_solver = [];

if sel.use_LOB
  opts_LOB = init_opts_LOB();
  opts_LOB.init_rank = init_rank;
  opts_LOB.max_iter = 100;
  opts_LOB.rtol = rtol;
  opts_LOB.lbfgsb_in.maxIter = 10;
  opts_solver.opts_LOB = opts_LOB;
end

if sel.use_DRTR
  opts_DRTR = init_opts_DRTR([d1, d2]);
  opts_DRTR.params.max_iter = 20;
  opts_DRTR.params.max_iter_tr = 200;
  opts_DRTR.params.vtol = 1e-3;
  opts_DRTR.params.rtol = rtol;
  opts_DRTR.params.check_dual = 0;
  opts_DRTR.model.p = init_rank;  
  opts_solver.opts_DRTR = opts_DRTR;
end

if sel.use_r1d
  opts_r1d = init_opts_LOB();
  opts_r1d.tol = 1;
  opts_r1d.L = 1e-2*10;
  opts_r1d.max_iter = 500;
  opts_solver.opts_r1d = opts_r1d;
end

if sel.use_SDP
  opts_SDP.init_rank = init_rank;
  opts_SDP.rtol = 1e-4;
  opts_SDP.max_iter = 500;
  opts_SDP.lstype = -1;
  opts_solver.use_SDP = use_SDP;
end


%% Run algorithm

exp_param.B = 9975;
exp_param.lambdas = {'1e-1'};
exp_param.perf_lab_y = 'Test regression error';

results = main_opt_singleview(exp_param, opts_solver, X0);

global osname

if strcmp(osname, 'linux')
  save([dataset '_' exp_param.lambdas{1} '.mat'], 'results');
  quit;
end
