% Use block coordinate descent on multiview dataset
%
function [sol, obj, res] = solve_BCD_mv(objf, lambda, X0, evalf, opts)
        
  % Partially corrective boosting
  % Solve  Min_X  func(X) + lambda * ||X||_mv

  [m, t] = size(X0);
  w1 = opts.prob_par.w1;
  w2 = opts.prob_par.w2;
  m1 = opts.prob_par.m1;  
  alpha = opts.prob_par.alpha;
  beta = opts.prob_par.beta;
  m2 = m - m1;
   
  k = opts.k;  % Initialize U and V with rank k

  res = struct('obj_list', [], ...
               'loss_list', [], ...
               'perf_list', [] ...
               );
  
  opts.fista_H_par.lambda = lambda;
  msg = 'Running';
  break_flag = 0;
  perf = struct('l1', 0, 'l2', 0);  
  par = opts.ori_par;
  obj = inf;
  
  time_list = [];
  time_H = [];
  time_B = [];
  time_C = [];
  
  prox_B = @(G, par)prox_op_L2ball(G, par, alpha);
  prox_C = @(G, par)prox_op_L2ball(G, par, beta);
  t1 = cputime;  
  B = randn(m1,k);
  B = B .* repmat( alpha ./ sqrt(sum(B.^2)), m1, 1);
  C = randn(m2,k);
  C = C .* repmat( beta ./ sqrt(sum(C.^2)), m2, 1);
  H = zeros(k, t);
    
  for i = 1 : opts.max_iter

    % First update H
    t2 = cputime;
    [H, f_opt_H, out] = minFISTA(@loss_H, @prox_op_L21, H, opts.fista_H_par);    
    time_H = [time_H; cputime - t2];
    
    t2 = cputime;
    [B, f_opt_B, out] = minFISTA(@loss_B, prox_B, B, opts.fista_B_par);
    time_B = [time_B; cputime - t2];
    
    t2 = cputime;
    [C f_opt_C, out] = minFISTA(@loss_C, prox_C, C, opts.fista_C_par);
    time_C = [time_C; cputime - t2];
    
    time_list = [time_list; cputime - t1];

    if ~isempty(evalf)
      perf = evalf([B*H; C*H]);
      res.perf_list = [res.perf_list; perf];
    end
    
    loss = w1*f_opt_B + w2*f_opt_C;
    fprintf('iter=%d, loss = %g, obj=%g, perf=(%g, %g), k=%d, time=%g\n', ...
            i, loss, f_opt_H, perf.l1, perf.l2, k, sum(time_list));
    
    res.loss_list = [res.loss_list; loss];    
    res.obj_list = [res.obj_list; f_opt_H];
          
    if i > 1
      if abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
        outputs.msg = 'Stop with small relative change';
        break_flag = 1;
      elseif sum(time_list) > opts.max_time
        outputs.msg = 'Stop by max_time';
        break_flag = 1;
      elseif i == opts.max_iter
        outputs.msg = 'Stop with max iteration';  
        break_flag = 1;
      end
      
      res.time_list = cumsum(time_list);
      res.time_H = cumsum(time_H);
      res.time_B = cumsum(time_B);
      res.time_C = cumsum(time_C);
      res.msg = msg;
      res.iter = i;

      if isfield(opts, 'res_fname')
        save(opts.res_fname, 'res', 'par');
      end

      if break_flag == 1, break;    end
    end
     
    pre_obj = obj;
    t1 = cputime;
  end

  sol.B = B;    sol.C = C;    sol.H = H;
  obj = f_opt_H;
  
  if opts.verbose
    fprintf(1, '%s\n', outputs.msg);
  end
  
    % local search objective
    function [f, g] = loss_B(BB)
      [f, G] = objf.obj_x(BB*H);
      g = G*H';
    end
  
    function [f, g] = loss_C(CC)     
      [f, G] = objf.obj_y(CC*H);
      g = G*H';
    end
  
    function [f, g] = loss_H(HH)
      [f1, G1] = objf.obj_x(B*HH);
      [f2, G2] = objf.obj_y(C*HH);
      g = w1*B'*G1 + w2*C'*G2;
      f = w1*f1 + w2*f2;
    end
  
end
